/***************************************************************************
 *   Copyright (C) 2011 by Dragos-Mircea Jeleriu <dragos.jeleriu@gmail.com>*
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "kwiki.h"
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <KLocale>

#include <plasma/svg.h>
#include <plasma/theme.h>

const int Spacing = 15;

KWiki::KWiki(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
    m_svg(this),
    m_icon("document")
{
    // this will get us the standard applet background, for free!
    setBackgroundHints(DefaultBackground);
    m_svg.setImagePath("widgets/background");
    setHasConfigurationInterface(true);    
    // Allow free resize
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    // Set some constraints to prevent breaking the layout
    setMinimumWidth(370);
    setMinimumHeight(220);
    setPreferredSize(370,450);
    // Set initial size
    resize(370,450);    
}


KWiki::~KWiki()
{
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
        
        // Cleanup
        wikiMainFrame->~Frame();
	wikiTopFrame->~Frame();
    }
}

void KWiki::init()
{
    // Initialize components
    wikiTopFrame = new Plasma::Frame(this);
    wikiMainFrame = new Plasma::Frame(this);
    wikiView = new Plasma::WebView(wikiMainFrame);
    wikiSearchTerm = new Plasma::LineEdit(wikiTopFrame);
    wikiSearchGo = new Plasma::PushButton(wikiTopFrame);
    
    // Setup the top frame, with the search menu
    wikiTopFrame->setFrameShadow(Plasma::Frame::Raised);
    wikiTopFrame->setMaximumHeight(9999);
    
    // Setup the content frame
    wikiMainFrame->setFrameShadow(Plasma::Frame::Raised);
    wikiMainFrame->setMaximumHeight(9999);
    
    // Setup the search field   
    wikiSearchTerm->setClickMessage("Type here to search ...");
    
    // Setup the search button
    wikiSearchGo->setText("Go");
    wikiSearchGo->setMinimumWidth(0);    
    
    // Assign the event handlers
    QObject::connect(wikiSearchGo,SIGNAL(clicked()),this,SLOT(wikiSearch()));
    QObject::connect(wikiSearchTerm,SIGNAL(returnPressed()),this,SLOT(wikiSearch()));
      
    // Set the start page to the mobile version of Wikipedia    
    wikiView->setUrl(KUrl("http://m.wikipedia.org"));     
}

void KWiki::wikiSearch()
{
  QString searchTerm = wikiSearchTerm->text();
  QString searchURL = "http://en.m.wikipedia.org/wiki?search="+searchTerm;
  
  // DEBUG CODE
  /*
   qDebug() << "Search called, the text is: " << searchTerm;
   qDebug() << "Search URL is: " << searchURL;
  */
  
  if ( searchTerm != "" ) {   
      wikiView->setUrl(KUrl(searchURL));
  }
}

void KWiki::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    wikiTopFrame->setGeometry(contentsRect.x(),contentsRect.y(),contentsRect.width(),40);
    wikiSearchTerm->setGeometry(
				QRectF(wikiTopFrame->contentsRect().x(),
				       wikiTopFrame->contentsRect().y(),
				       wikiTopFrame->contentsRect().width()-40,
				       wikiTopFrame->contentsRect().height()
				      )
			       );
    
    wikiSearchGo->setGeometry(
				QRectF(wikiSearchTerm->boundingRect().right()+5,
				       wikiTopFrame->contentsRect().y(),
				       wikiTopFrame->contentsRect().width() - wikiSearchTerm->contentsRect().width(),
				       wikiTopFrame->contentsRect().height()
				      )
			     );
   
    
    wikiMainFrame->setGeometry(contentsRect.x(),
			       wikiTopFrame->rect().height()+Spacing,
			       contentsRect.width(),
			       contentsRect.height()-(wikiTopFrame->boundingRect().height())
			      );
    wikiView->setGeometry(wikiMainFrame->contentsRect());
    
    // DEBUG CODE
    /*
     qDebug() << "Window Size:" << contentsRect;
     qDebug() << "Main Frame Size:" << wikiMainFrame->rect();
    */
}

#include "kwiki.moc"
